"""
Provides functions for probing / NER that can be called with model name and the required data.
Makes evaluating multiple models easier.
"""
from src.models.LLM_interface import available_models as llms
from src.models.GLiNER import find_entities as find_entities_gliner
from src.models.GLiNER import classify_entity as classify_entity_gliner
from src.models.T5_NLI import classify_entity as classify_entity_t5_nli
from src.models.T5_MLM_label import classify_entity as classify_entity_t5_mlm_label
from src.models.T5_MLM_entity import classify_entity as classify_entity_t5_mlm_entity
from src.models.T5_MLM_entity import set_label_dict as set_label_dict_t5_mlm_entity
from src.models.Word2Vec import classify_entity as classify_entity_word2vec
from src.models.Word2Vec import set_label_dict as set_label_dict_word2vec
from src.experiments.NER_with_LLMs.NER_with_LLMs import find_entities as find_entities_llm
from src.experiments.NER_with_LLMs.NER_with_LLMs import classify_entity as classify_entity_llm


def classify_entity(model_name, sentence, entity, labels):
    """
    NEC. Returns label (string) for entity.
    """
    if model_name == "T5-NLI":
        return classify_entity_t5_nli(sentence, entity, labels)
    elif model_name == "T5-MLM-label":
        return classify_entity_t5_mlm_label(sentence, entity, labels)
    elif model_name == "T5-MLM-entity":
        return classify_entity_t5_mlm_entity(sentence, entity, labels)
    elif model_name == "GLiNER":
        return classify_entity_gliner(sentence, entity, labels)
    elif model_name == "Word2Vec":
        return classify_entity_word2vec(entity, labels)
    elif model_name in llms:
        return classify_entity_llm(model_name, sentence, entity, labels)
    else:
        print(f"classify_entity not implemented for {model_name}")


def find_entities(model_name, sentence, labels):
    """
    NER. Returns list of pairs [(entity, label), ...]
    """
    if model_name in llms:
        return find_entities_llm(model_name, sentence, labels)
    elif model_name == "GLiNER":
        return find_entities_gliner(sentence, labels)
    else:
        print(f"find_entities not implemented for {model_name}")


def set_label_dict(model_name, label_dict):
    """
    Sets the label dictionary required for the Word2Vec model
    """
    if model_name == "Word2Vec":
        return set_label_dict_word2vec(label_dict)
    elif model_name == "T5-MLM-entity":
        return set_label_dict_t5_mlm_entity(label_dict)
    else:
        print(f"set_label_dict not implemented for {model_name}")
