"""
This module implements the finetuning procedure for the MLM label task formulation for the named entity classification task.
By default, the model is finetuned on the FIGER-coarse dataset with 1000 example sentences.
"""

import data.data_manager as data_manager
from src.models.T5_MLM_label import finetune_model


def finetune_t5(dataset):
    print("start")
    annotated_sentences = data_manager.get_annotated_sentences(dataset, 2000, train_test_split=0.5, train=True)

    sentences = []
    entities = []
    labels = []

    for annotated_sentence in annotated_sentences:
        sentence = annotated_sentence[0]

        for annotation in annotated_sentence[1]:
            sentences.append(sentence)
            entities.append(annotation[0])
            labels.append(annotation[1])

    print(f"Finetuning on {len(sentences)} examples")

    for i in range(min(len(sentences), 50)):
        print(f"sentence: {sentences[i]}, entity: {entities[i]}, label: {labels[i]}")

    epochs = 150

    finetune_model(sentences, entities, labels,
                   output_dir=f"./src/models/t5_mlm_label_finetuned_model/pretrained_{dataset}_epoch{epochs}",
                   epochs=epochs)


finetune_t5("FIGER-coarse")
