"""
This module implements the finetuning procedure for the NLI task formulation for the named entity classification task.
By default, the model is finetuned on the FIGER-coarse dataset with 1000 example sentences.
"""

import data.data_manager as data_manager
from src.models.T5_NLI import finetune_model


def finetune_t5(dataset):
    print("start")
    annotated_sentences = data_manager.get_annotated_sentences(dataset, 2000, train_test_split=0.5, train=True)
    labels = data_manager.get_labels(dataset)

    premises = []
    hypotheses = []
    entailment = []

    for annotated_sentence in annotated_sentences:
        sentence = annotated_sentence[0]

        for annotation in annotated_sentence[1]:
            entity = annotation[0]
            for label in labels:
                premises.append(sentence)
                hypotheses.append(f"{entity} is a {label}")
                entailment.append(label == annotation[1])

    print(f"Finetuning on {len(premises)} examples")

    for i in range(min(len(premises), 50)):
        print(f"premise: {premises[i]}, hypothesis: {hypotheses[i]}, entailment: {entailment[i]}")

    epochs = 150

    finetune_model(premises, hypotheses, entailment,
                   output_dir=f"./src/models/t5_nli_finetuned_model/pretrained_{dataset}_epoch{epochs}", epochs=epochs)


finetune_t5("FIGER-coarse")
