"""
This module is used to plot the loss curve of the T5 finetuning tasks. The loss values are extracted from the Slurm output file.
To use this module, the correct filenames must be substituted in the call to `plot_loss_curve`
"""

import os
import re
import pandas as pd
import matplotlib.pyplot as plt

def plot_loss_curve(logfile, title):

    eval_losses = []
    pattern = re.compile(r"'eval_loss': ([\d\.e-]+)")
    
    with open(logfile, 'r', errors='ignore') as file:
        for line in file:
            match = pattern.search(line)
            if match:
                eval_losses.append(float(match.group(1)))

    plt.figure(figsize=(10,5))
    plt.plot(eval_losses, label="Eval Loss")

    plt.xlabel("Epoch")
    plt.ylabel("Eval Loss")
    plt.title(title)
    plt.legend()
    plt.grid(True)

    plt.savefig(f"eval_loss_{os.path.basename(logfile)}.svg")

plot_loss_curve("logs/finetune_T5_MLM_entity_427082.txt", "T5 Finetuning - MLM Entity Masking")
plot_loss_curve("logs/finetune_T5_MLM_label_427081.txt", "T5 Finetuning - MLM Label Masking")
plot_loss_curve("logs/finetune_T5_NLI_427080.txt", "T5 Finetuning - NLI")
