"""
This module implements the Word2Vec based approach for the named entity recogition task. It relies on the label dictionary functionality of the data manager for the required representative entities.
"""

from gensim.models import Word2Vec
import gensim.downloader as api
import string


def train(sentences):
    global model
    # make lowercase, remove punctuation, split at whitespace
    sentences = [sentence.lower().translate(str.maketrans('', '', string.punctuation)).split() for sentence in
                 sentences]
    model = Word2Vec(vector_size=100, window=5, min_count=1, workers=4)
    model.build_vocab(sentences)
    model.train(sentences, total_examples=model.corpus_count, epochs=model.epochs)

    print(model.wv.most_similar("him"))


def load_pretrained():
    global model
    print("Loading model: Word2Vec Pretrained Google News")
    model = Word2Vec(vector_size=300, window=5, min_count=1)
    model.wv = api.load('word2vec-google-news-300')
    model.trainables = None
    print("Finished loading model: Word2Vec Pretrained Google News")


def similarity(word1, word2):
    if word1 in model.wv.key_to_index and word2 in model.wv.key_to_index:
        return model.wv.similarity(word1, word2)
    else:
        return float("inf")


def print_nearest(word):
    global model
    if word in model.wv.key_to_index:
        print(model.wv.most_similar("him"))
    else:
        print("word is not in vocabulary")


# dictionary containing example values for each label
def set_label_dict(label_dict):
    global label_representatives
    label_representatives = label_dict


def classify_entity(entity, labels):
    global label_representatives
    best_label = None
    best_similarity = float("inf")

    # Keep track how many of the entities have word vectors
    has_vecs = 0
    has_no_vecs = 0

    # Find label with highest probability of entailment
    for label in labels:
        representatives = label_representatives[label]

        # do not cheat if entity is one of the representatives already
        if entity in representatives:
            representatives.remove(entity)

        best_for_label = float("inf")

        for representative in representatives:
            sim = similarity(entity, representative)
            if sim == float("inf"):
                has_no_vecs += 1
            else:
                has_vecs += 1

            best_for_label = min(best_for_label, sim)

        if best_for_label < best_similarity:
            best_similarity = best_for_label
            best_label = label

    # print(f"{has_vecs} entities had word vectors, {has_no_vecs} entities did not have word vectors")

    return best_label if best_label else labels[0]

load_pretrained()
