"""
Calls all the available large language models and tests their functionality.
"""
import time
from src.models.LLM_interface import LLM, available_models
from huggingface_hub import login
from dotenv import load_dotenv
import os


load_dotenv()
access_token = os.getenv('HF_ACCESS_TOKEN')
if access_token:
    login(token=access_token)
else:
    raise ValueError("Hugging Face access token not found.")


def test_all():
    for m in available_models:
        try:
            model = LLM.create(m, "This is a test.")  # System prompt
            start = time.time()
            print(f"{m}: " + model("Respond with the sentence \"All systems functional.\"."))
            print(f"{m}: " + model("What was the first prompt I gave you?", reset=False))  # Test whether chat history works
            end = time.time()
            print(f"{m} test duration: {end - start}s")
            if hasattr(model, "token_usage") and isinstance(model.token_usage, list):
                total_prompt_tokens = sum(usage["prompt_tokens"] for usage in model.token_usage)
                total_completion_tokens = sum(usage["completion_tokens"] for usage in model.token_usage)
                total_tokens = sum(usage["total_tokens"] for usage in model.token_usage)

                print("Total prompt tokens: " + str(total_prompt_tokens))
                print("Total completion tokens: " + str(total_completion_tokens))
                print("Total tokens: " + str(total_tokens))
        except Exception as e:
            print(f"Error occurred while testing {m}: {e}")


test_all()
